#!/usr/bin/env python

import json
import logging
import logging.handlers
import os, sys
import ovh
import requests
import subprocess
import time
import yaml
from dotenv import load_dotenv

my_logger = logging.getLogger('MyLogger')
my_logger.setLevel(logging.DEBUG)

handler = logging.handlers.SysLogHandler(address = '/dev/log')
formatter = logging.Formatter("dyndns-update: %(message)s")
handler.setFormatter(formatter)

my_logger.addHandler(handler)


env_path = os.path.abspath ( os.path.join ( os.curdir, os.path.relpath('.env') ) )
load_dotenv(verbose=True, dotenv_path=env_path)

## constants
client = ovh.Client(
    endpoint=os.getenv('ENDPOINT'),               # Endpoint of API OVH Europe (List of available endpoints)
    application_key=os.getenv('APP_KEY'),    # Application Key
    application_secret=os.getenv('SECRET'), # Application Secret
    consumer_key=os.getenv('CONSUMER_KEY'),       # Consumer Key
)

api_token=os.getenv('API_TOKEN')
provider = 'hetzner'
TMPFILE = '/tmp/lastIP'

rid=None
zid=None

WEBRTCCONFIGFILE = '/etc/bigbluebutton/bbb-webrtc-sfu/production.yml'
# WEBRTCCONFIGFILE = 'test.yml'

def get_ip():
    r = requests.get('http://ipinfo.io').json()
    return (r['ip'])

def get_zone_id(zone):
    if zid != None: return zid

    try:
        response = requests.get(
            url="https://dns.hetzner.com/api/v1/zones",
            headers={
                "Auth-API-Token": api_token,
            }
        ).json()
    except requests.exceptions.RequestException:
        my_logger.critical('HTTP Request failed')
        exit(1)

    id = [z.get('id') for z in response.get('zones') if z.get('name')==zone]

    rid=id[0]
    return rid


def get_record_id(zone,subdomain):
    if rid != None: return rid

    if provider == 'ovh':
        id = client.get('/domain/zone/' + zone + '/record', fieldType='A', subDomain=subdomain )

    else:
      try:
        response = requests.get(
          url="https://dns.hetzner.com/api/v1/records",
          params={
            "zone_id": get_zone_id(zone),
          },
          headers={
            "Auth-API-Token": api_token,
          }
        ).json()

        id = [rec.get('id') for rec in response.get('records') if rec.get('name')== subdomain]

      except requests.exceptions.RequestException:
        my_logger.critical('HTTP Request failed')
        exit(1)

    zid = id[0]
    return zid


def set_record(zone, subdomain, ip):
    if provider == 'ovh':
        rid=get_record_id(zone,subdomain)
        result = client.put('/domain/zone/' + zone + '/record/' + str(rid), subDomain=subdomain, target=rip, ttl=60)

        if result != None:
            my_logger.critical('couldn\'t change record')
            exit(1)

        result = client.post('/domain/zone/' + zone + '/refresh')
        return result
    else:
        try:
            response = requests.put(
                url="https://dns.hetzner.com/api/v1/records/" + get_record_id(zone,subdomain),
                headers={
                    "Content-Type": "application/json",
                    "Auth-API-Token": api_token,
                },
                data=json.dumps({
                    "value": ip,
                    "ttl": 60,
                    "type": "A",
                    "name": subdomain,
                    "zone_id": get_zone_id(zone)
                })
            )
            return (None if response.status_code == requests.codes.ok else False)
        except requests.exceptions.RequestException:
            my_logger.critical('HTTP Request failed')
            exit(1)


def read_config_file():
    with open(WEBRTCCONFIGFILE,'r') as file:
        webrtc_config = yaml.safe_load(file)
    return webrtc_config


def write_config_file(ip):
    webrtc_config['mediasoup']['webrtc']['listenIps'][0]['announcedIp']=ip
    webrtc_config['mediasoup']['plainRtp']['listenIp']['announcedIp']=ip
    webrtc_config['freeswitch']['ip']=ip

    with open(WEBRTCCONFIGFILE, 'w') as file:
        yaml.dump(webrtc_config, file)


if len(sys.argv) > 2:
    webrtc_config = read_config_file()

    lastIP = webrtc_config['freeswitch']['ip']
    currentIP = get_ip()

    if lastIP == currentIP:
        my_logger.debug ('IP has not changed since last call')
        exit(1)

    if (set_record  (sys.argv[1], sys.argv[2], currentIP) == None ):
        write_config_file(currentIP)
        
        subprocess.run(["ip","a","del",lastIP + '/32',"dev","lo"])
        subprocess.run(["ip","a","add",currentIP,"dev","lo"])

        my_logger.debug('IP of ' + sys.argv[2] + '.' + sys.argv[1] + ' successfully changed to ' + currentIP)
    else:
        my_logger.critical('Couldn\'t change IP')


else:
  print('Updates IP record of SUBDOMAIN in ZONE to current IP from ipinfo.io\n\nUsage:\n' + sys.argv[0] + ' ZONE SUBDOMAIN')
